@@ -1,3 +1,5 @@
+import copy
+import logging
 import os
 import json
 import argparse
@@ -37,31 +39,34 @@ def get_payload_data(request, requested_fields):
                                 nel caso venga sollevata un'eccezione, conterrà una stringa contenente il dizionario da restituire al chiamante
     :return:                    dizionario contenente i dati
     """
+
+    verbose = cherrypy.config.get('lisp.verbose_errors_responses', False)
+
     try:
         # ottengo la dimensione del payload
         content_length = int(request.headers['Content-Length'])
 
         # ottengo la dimensione massima ammissibile (definita ad hoc) dalla configurazione
-        max_request_size = cherrypy.config.get('lisp.request.max_size_mb') * 1000**2
+        max_request_size = cherrypy.config.get('lisp.request.max_size_mb', 2) * 1000**2
 
         # controllo sulla dimensione massima ammissibile
         if content_length > max_request_size:
             # la dimensione della richiesta supera la dimensione massima consentita
-            raise RuntimeError(build_response(status=1, substatus=1, message="Payload content-length greater than maximum allowed", data=max_request_size))
+            raise RuntimeError(build_response(status=1, substatus=1, message="Payload content-length greater than maximum allowed", data="Max size: {} MB".format(max_request_size) if verbose else ''))
 
         # per ovviare al fatto che la grandezza del payload potrebbe non coincidere con quella dichiarata nel content-length,
         # andiamo a leggere solo content_length dati
         payload = json.loads(request.body.read(content_length))
     except json.JSONDecodeError:
         # errore nella decodifica del json
-        raise RuntimeError(build_response(status=1, substatus=2, message="Json content cannot be parsed"))
+        raise RuntimeError(build_response(status=1, substatus=2, message="Json content cannot be parsed" if verbose else ''))
     except ValueError:
         # errore nel casting per il content_length
-        raise RuntimeError(build_response(status=1, substatus=3, message="Content-length not specified or not valid integer"))
+        raise RuntimeError(build_response(status=1, substatus=3, message="Content-length not specified or not valid integer" if verbose else ''))
 
     # controllo che tutti i campi richiesti siano presenti nel payload
     if not all(k in payload for k in requested_fields):
-        raise RuntimeError(build_response(status=1, substatus=4, message="Missing fields in JSON", data=requested_fields))
+        raise RuntimeError(build_response(status=1, substatus=4, message="Missing fields in JSON" if verbose else '', data=requested_fields if verbose else None))
 
     return payload
 
@@ -119,22 +124,36 @@ def create_config_dir(path):
         raise RuntimeError('Something went wrong trying to create example files. Error: {}'.format(e))
 
 
-def handle_error():
+def default_error_handler(status, message, traceback, version):
     """
-    Funzione per la mascheratura degli errori non previsti del server
-    Questa funzione serve ad evitare che venga ritornato un 500 al client
+    Funzione per la mascheratura degli errori del server (500, etc)
     """
     # gli errori vengono loggati in automatico da cherrypy
 
+    logging.info("Default error handler called: status={}, message='{}', traceback:{}".format(status, message, traceback))
+
     # imposto lo stato della risposta
     cherrypy.response.status = 500
 
     # imposto il contenuto della risposta
-    cherrypy.response.body = [
-        bytes(json.dumps({'status': 2, 'substatus': 0, 'message': 'Oops, something went wrong'}), encoding='utf-8')
-    ]
+    return json.dumps({'status': 2, 'substatus': 0, 'message': 'Oops, something went wrong', 'data': None})
+
     # volendo si possono fare altre azioni come inviare un email
 
+def err_404_error_handler(status, message, traceback, version):
+    """
+    Funzione per gestire gli errori 404
+    """
+    # gli errori vengono loggati in automatico da cherrypy
+
+    logging.info("Default error handler called: {} {} {}".format(status, message, traceback))
+
+    # imposto lo stato della risposta
+    cherrypy.response.status = 404
+
+    # imposto il contenuto della risposta
+    return json.dumps({'status': 2, 'substatus': 0, 'message': 'The requested path does not exist.'})
+
 
 def load_apikey_configs(apikey_folder):
     """
@@ -171,4 +190,29 @@ def load_apikey_configs(apikey_folder):
         return deployers_dict
     except (ValueError, FileNotFoundError, RuntimeError) as e:
         # try/except esterno al for in modo da bloccare il funzionamento se esiste anche un'unica config errata
-        raise RuntimeError('An error occurred loading api key configs. Error {}: {}'.format(type(e), e))
\ No newline at end of file
+        raise RuntimeError('An error occurred loading api key configs. Error {}: {}'.format(type(e), e))
+
+
+def log_api_call(request, data, result):
+    l = logging.getLogger('app')
+
+    if isinstance(data, dict):
+        # evito di modifcare il dizionario del chiamante
+        data = data.copy()
+        # se è presente un api key la tolgo
+        if 'apikey' in data:
+            data['apikey'] = '(omitted)'
+
+        data = json.dumps(data, default=str)
+
+    if len(data) > 300:
+        data = data[:300] + '...(truncated)'
+
+    if isinstance(result, dict):
+        result = json.dumps(result, default=str)
+
+    if len(result) > 300:
+        result = result[:300] + '...(truncated)'
+
+
+    l.debug('{} {} {} {}'.format(request.remote.ip, cherrypy.url(qs=cherrypy.request.query_string, relative="server"), data, result))
